#include "EntBomb.h"
#include "Map.h"
#include "Sound.h"
#include "GameLogic.h"

#define C_NUMBER_OF_BOMB_SOUNDS 5

EntBomb::EntBomb()
{

}

EntBomb::~EntBomb()
{
	g_Map.GetTile(m_tileID)->RemoveBomb();

}

bool EntBomb::Init(int tileID, int fireReachSpeedMS, int tileReach)
{
	LogMsg("Initting bomb.");

	//setup some defaults

	m_countDownTimer = 2500;
	m_explosionReach = tileReach; //in tiles
	m_tileID = tileID;
	m_fireReachSpeedMS = fireReachSpeedMS;

	SAFE_DELETE(m_sprite);
	m_sprite = new CL_Sprite("bomb", GetApp()->GetResourceManager());

	SetPos(g_Map.TileToPixelPos(tileID));
	
	m_alpha = 0.3;
	m_vScale = CL_Vector2(4,4);
	
	m_state = STATE_DROPPING;
	g_Sound.Play("drop.wav");
	m_dropTimer = GetApp()->GetGameTick()+1000;
	g_Map.GetTile(m_tileID)->SetBomb(this);


	return true;

}

void EntBomb::Diffuse()
{
	if (m_state == STATE_TICKING)
	{
		m_bScaleToNothing = true;
		GetMessageDispatcher()->SendMsg(C_MESSAGE_KILL_ENTITY, 150, this);
		m_state = STATE_DIFFUSED;
		g_Sound.Play("diffuse.wav");

		int points = 25;
		GetGameLogic()->ModScore(points);

		Say(CL_String::from_int(points)+" pts", this, 0);

	}
	
}

void EntBomb::StartTicking()
{
	m_state = STATE_TICKING;
	m_timeAtStart = GetApp()->GetGameTick();

}

float EntBomb::GetSecondsTillExplosion()
{
	if (m_state != STATE_TICKING)
	{
		return 50;
	}
	
	return float(((m_timeAtStart+m_countDownTimer)  - GetApp()->GetGameTick())) /float(1000);
}

void EntBomb::Render()
{
	
	if (m_state != STATE_EXPLODING)
	{
		float secs = GetSecondsTillExplosion();
		//LogMsg("Secs: %.2f", secs);
		if (secs > 1.2)
		{
			m_sprite->set_frame(0);
		} else if (secs > 0.4)
		{
			m_sprite->set_frame( GetApp()->GetGameTick()%300 > 150);
		} else
		{
			
			m_sprite->set_frame( GetApp()->GetGameTick()%100 > 50);
		}
		CL_Vector2 vTemp = m_vPosition;
		m_vPosition += CL_Vector2(C_TILE_SIZE/2, C_TILE_SIZE/2);

		Entity::Render();
		m_vPosition = vTemp;
	}

	/*
	switch (m_state)
	{
	case STATE_TICKING:
		CL_Vector2 v =g_Map.TileToPixelPosCentered(m_tileID);
		GetApp()->GetFont(C_FONT_SMALL)->draw(v.x, v.y, CL_String::from_int(GetSecondsTillExplosion()));

		break;
	}
	*/
}

void EntBomb::SetFireTo(CL_Vector2 vTilePos, eDir dir)
{
	if (g_Map.IsValid(vTilePos))
	{
		if (g_Map.GetTile(vTilePos)->GetObstacle())
		{
			if (!g_Map.GetTile(vTilePos)->GetObstacle()->FireCanPassThrough())
			{
				m_reachDirActive[dir] = false;
			}
		}
		g_Map.GetTile(vTilePos)->SetOnFire();
	}
}

void EntBomb::ProcessExplosion()
{

	if (m_curReach == 0)
	{
		int r = Random(C_NUMBER_OF_BOMB_SOUNDS-1);
		g_Sound.Play("bomb"+ CL_String::from_int(r)+".wav");
	}

	CL_Vector2 v = g_Map.TileToPos(m_tileID);

	for (int i=0; i < C_DIR_COUNT; i++)
	{
		if (m_reachDirActive[i])
		{
			CL_Vector2 vReach = DirStep(eDir(i))*m_curReach;
			SetFireTo(v+vReach,eDir(i));

		}
	}

	//when to call it again
	m_nextProcessTime = GetApp()->GetGameTick() + m_fireReachSpeedMS;
	m_curReach++;

	if (m_curReach > m_explosionReach)
	{
		//done exploding..
		SetDeleteFlag(true);
	}

}
void EntBomb::Update( float step )
{
	Entity::Update(step);

	switch (m_state)
	{
	case STATE_DROPPING:

		if (m_dropTimer < GetApp()->GetGameTick())
		{
			m_alpha = 1;
			m_vScale = CL_Vector2(1,1);
				 
			StartTicking();
		}

		m_alpha = SetFloatWithTarget(m_alpha, 1, 0.026*step);
		m_vScale.x = SetFloatWithTarget(m_vScale.x, 1, 0.036*step);
		m_vScale.y = SetFloatWithTarget(m_vScale.y, 1, 0.036*step);


		break;
	
	case STATE_TICKING:
		
		if (GetSecondsTillExplosion() <= 0 || g_Map.GetTile(m_tileID)->IsOnFire())
		{
			m_state = STATE_EXPLODING;
			m_curReach = 0;
			
			for (int i=0; i < C_DIR_COUNT; i++)
			{
				m_reachDirActive[i] = true;
			}

			ProcessExplosion();
		}

		break;
	
	case STATE_EXPLODING:

		if (m_nextProcessTime < GetApp()->GetGameTick())
		{
			ProcessExplosion();
		}
		break;

	}
}