#include "EntGirl.h"
#include "Sound.h"
#include "Map.h"
#include "GameLogic.h"
#include "EntPlayer.h"

#define C_BASE_POINTS 50

EntGirl::EntGirl()
{
	m_bHaveClearedMapTile = false;
}

EntGirl::~EntGirl()
{
	if (!m_bHaveClearedMapTile)
	g_Map.GetTile(m_tileID)->SetGirl(NULL);
	
	GetGameLogic()->ModGirlsLeft(-1);

	if (!GetLevelLogic()->IsPaused())
	{
		if (GetGameLogic()->GetGirlsLeft() == 0)
		{
			
			//I guess we finished the level
			GetMessageDispatcher()->SendMsg(C_MESSAGE_INIT_END_LEVEL_TEXT, 50);
			GetLevelLogic()->SetPaused();
		}
	}

	ResetThinkTimer();
}

void EntGirl::ResetThinkTimer()
{
	m_thinkTimer = GetApp()->GetGameTick()+1000+Random(500);
}

bool EntGirl::Init(int tileID)
{
	m_sprite = new CL_Sprite("kid1", GetApp()->GetResourceManager());
	m_tileID = tileID;
	m_vPosition = g_Map.TileToPixelPos(m_tileID);
	g_Map.GetTile(m_tileID)->SetGirl(this);
	m_curAnim = ANIM_IDLE;
	m_curFrame = 0;
	m_nextFrameTimer = 0;
	m_animSpeedMS = 250;
	m_totalFramesInAnim = 2;
	m_pointValue = 100;
	m_bHaveClearedMapTile = false;
	SetPointIncreaseTimer();
	m_timesRaised = 0;
	GetGameLogic()->ModGirlsLeft(1);
	return true;
}

void EntGirl::SetPointIncreaseTimer()
{
	m_pointIncreaseTimer = GetApp()->GetGameTick() + 1000;
}

void EntGirl::OnRescued()
{
	GetMessageDispatcher()->SendMsg(C_MESSAGE_FADE_OUT_AND_DIE, 0, this, 0);
	GetMessageDispatcher()->SendMsg(C_MESSAGE_KILL_ENTITY, 250, this);
	m_bScaleToNothing = true;
	g_Sound.Play("pickup.wav");
	g_Map.GetTile(m_tileID)->SetGirl(NULL);
	m_bHaveClearedMapTile = true;

	GetGameLogic()->ModScore(m_pointValue);

		
	int r = Random(4);

	string text;
	
	if (m_curAnim == ANIM_IDLE)
	{
		switch (r)
		{
		case 0: text = "Thankyou!"; break;
		case 1: text = "My hero!"; break;
		case 2: text = "I love you!"; break;
		case 3: text = "I'm saved!"; break;
		case 4: text = "Praise Jeebus!"; break;
		}

	} else
	{
		//they are dead
		switch (r)
		{
		case 0: text = "About time, dickwad!"; break;
		case 1: text = "A little slower perhaps?"; break;
		case 2: text = "Am I girl or a crisp?"; break;
		case 3: text = "I'll never model again, jackass!"; break;
		case 4: text = "Somehow hair my hair survived.  Wow."; break;
		}
	}

	if (Random(20) == 0)
	Say(text, this, 50); else
	{
		Say(CL_String::from_int(m_pointValue)+" pts", this, 0);
	}
}

void EntGirl::Update(float step)
{
	Entity::Update(step);
	
	if (m_nextFrameTimer < GetApp()->GetGameTick())
	{
		m_nextFrameTimer = GetApp()->GetGameTick()+ m_animSpeedMS;
		m_curFrame = ++m_curFrame%m_totalFramesInAnim;
	}


	if (m_bScaleToNothing) return;
	
		if (g_Map.GetTile(m_tileID)->IsOnFire() && m_curAnim == ANIM_IDLE)
		{
			//we burn!
			OnBurn();
		} else
		{

		
			/*
			if (m_curAnim == ANIM_IDLE)
				if (m_pointIncreaseTimer < GetApp()->GetGameTick())
				{
					m_pointValue += C_BASE_POINTS+( (C_BASE_POINTS *m_timesRaised)*0.10);
					m_timesRaised++;
					SetPointIncreaseTimer();
				}
				*/

			if (m_curAnim == ANIM_DEAD)
			{
				//fade out slowly
				m_alpha -= 0.002f*step;

				if (m_alpha < 0.1)
				{
					SetDeleteFlag(true);
				}

			}

			if (m_thinkTimer < GetApp()->GetGameTick())
			{
				ResetThinkTimer();
				if (m_curAnim == ANIM_IDLE)
				{
					
					int r = Random(4);

					string text;
					switch (r)
					{
					case 0: text = "Over here!"; break;
					case 1: text = "Please!"; break;
					case 2: text = "Can you hear me?!"; break;
					case 3: text = "<sob>"; break;
					}

					if (Random(10) == 0)
					{
						if (g_Map.GetDistanceToPlayer(m_tileID) > 7)					
						{
							Say(text, this, 50);
						}
					}

					
				}
			}
				
		}
}

void EntGirl::OnBurn()
{
	m_curAnim = ANIM_DEAD;
	g_Sound.Play("burn.wav");

	int r = Random(4);

	string text;
	switch (r)
	{
	case 0: text = "It burns!!!"; break;
	case 1: text = "Mommie!"; break;
	case 2: text = "Aaaarghghg!"; break;
	case 3: text = "Goddamnit!"; break;
	}

	if (Random(10) == 0)
	Say(text, this, 50);

	m_pointValue = 50;

}

void EntGirl::Render()
{
	
	
	m_sprite->set_frame(m_curAnim+m_curFrame);
	
	CL_Vector2 vTemp = m_vPosition;
	m_vPosition += CL_Vector2(C_TILE_SIZE/2, C_TILE_SIZE/2);

	Entity::Render();
	m_vPosition = vTemp;


	//CL_Vector2 v =g_Map.TileToPixelPosCentered(m_tileID);
	//GetApp()->GetFont(C_FONT_SMALL)->draw(v.x, v.y-32, CL_String::from_int(m_pointValue)+" pts");

}
