#include "EntObstacle.h"
#include "GameLogic.h"

#include "Sound.h"
#include "Map.h"

EntObstacle::EntObstacle()
{
}

bool EntObstacle::Init(int tileID, eObstacleType type)
{
	m_sprite = new CL_Sprite("obstacle", GetApp()->GetResourceManager());
	m_tileID = tileID;
	m_type = type;
	m_vPosition = g_Map.TileToPixelPos(m_tileID);
	g_Map.GetTile(m_tileID)->SetObstacle(this);
	
	if (IsDestroyedByFire())
	{
		m_bScaleUp = true;
		m_vScale = CL_Vector2(0,0);
	}


	m_bDying = false;
	return true;
}

void EntObstacle::BurnedByFire()
{
	if (IsDestroyedByFire())
	{
		SetDeleteFlag(true);
	}
}

bool EntObstacle::FireCanPassThrough()
{
	switch (m_type)
	{
	case OBSTACLE_WATER:
		return true;
	}

	return false;
}

void EntObstacle::AttemptToDestroy()
{

	if (m_bDying) return;

	switch (m_type)
	{
	case OBSTACLE_WOOD:
		m_bDying = true;	
		//sure, we can break this
		GetMessageDispatcher()->SendMsg(C_MESSAGE_FADE_OUT_AND_DIE, 0, this, 0);
		GetMessageDispatcher()->SendMsg(C_MESSAGE_KILL_ENTITY, 150, this);
		m_bScaleToNothing = true;
		g_Sound.Play("destroy.wav");
		break;
	}
}

EntObstacle::~EntObstacle()
{
	g_Map.GetTile(m_tileID)->SetObstacle(NULL);
}

void EntObstacle::Render()
{
	m_sprite->set_frame(m_type);
	
	CL_Vector2 vTemp = m_vPosition;
	m_vPosition += CL_Vector2(C_TILE_SIZE/2, C_TILE_SIZE/2);

	Entity::Render();
	m_vPosition = vTemp;
}

bool EntObstacle::IsBurnable()
{
	switch (m_type)
	{
	case OBSTACLE_ROCK: return false; break;
	}

	return true;
}


bool EntObstacle::IsDestroyedByFire()
{
	switch (m_type)
	{
		case OBSTACLE_WOOD: return true; break;
	}

	return false;
}