
#include "EntPlayer.h"
#include "GameLogic.h"
#include "Map.h"
#include "Sound.h"
#include "EntObstacle.h"

#define C_PLAYER_MOVE_DELAY_MS 200
#define C_MOVE_PIXELS 5

EntPlayer::EntPlayer()
{ 
	m_lives = GetLevelLogic()->GetLives();
	for (int i=0; i < C_DIR_COUNT; i++)
	{
		m_spriteDir[i] = NULL;	
		m_dirTimer[i] = 0;
	}
	m_invicibilityTimer = 0;
	m_spriteDead = NULL;
	m_spriteLastUsed = NULL;
	m_state = STATE_NORMAL;	
	ResetKeys();
	
	m_SlotKeyUp = CL_Keyboard::sig_key_up().connect( this, &EntPlayer::OnKeyUp);
	m_SlotKeyDown = CL_Keyboard::sig_key_down().connect( this, &EntPlayer::OnKeyDown);
	m_bInputKilled = false;
	SetPosTile(0);
	if (!Init()) throw ("Error initting player");
	m_ignoreKeysUntilTimer = GetApp()->GetGameTick()+1000;

	
}

EntPlayer::~EntPlayer()
{
	for (int i=0; i < C_DIR_COUNT; i++)
	{
		SAFE_DELETE(m_spriteDir[i]);
	}
}

void EntPlayer::SetPosTile(int tileID)
{
	m_destTileID = m_activeTileID = tileID;
	m_vPosition = g_Map.TileToPixelPos(tileID);
}

void EntPlayer::ResetKeys()
{
	m_Keys = 0; 
}

bool EntPlayer::Init()
{
	//setup stuff
	m_spriteDir[C_DIR_DOWN] = new CL_Sprite("hero_down", GetApp()->GetResourceManager());
	m_spriteDir[C_DIR_UP] = new CL_Sprite("hero_up", GetApp()->GetResourceManager());
	m_spriteDir[C_DIR_LEFT] = new CL_Sprite("hero_left", GetApp()->GetResourceManager());
	m_spriteDir[C_DIR_RIGHT] = new CL_Sprite("hero_right", GetApp()->GetResourceManager());
	
	m_spriteDead = new CL_Sprite("hero_dead", GetApp()->GetResourceManager());

	m_sprite = new CL_Sprite();
	m_lastDir = C_DIR_RIGHT;
	SetSpriteData(m_lastDir);
	return true; //success
}
void EntPlayer::KillInput()
{
	
	if (!m_bInputKilled)
	{
		m_bInputKilled = true;

		//Don't know why CL isn't liking this.  Will do another work around..
	//	CL_Keyboard::sig_key_up().disconnect(m_SlotKeyUp);
	//	CL_Keyboard::sig_key_down().disconnect(m_SlotKeyDown);

	}
}

void EntPlayer::ResetMove(eDir dir)
{
	//m_dirTimer[dir] = 0;
	m_lastDir = dir;
}

void EntPlayer::OnKeyDown(const CL_InputEvent &key)
{
	
	if (m_ignoreKeysUntilTimer > GetApp()->GetGameTick()) return;

	if (m_state == STATE_DEAD || GetLevelLogic()->IsPaused())
	{

		if (GetLevelLogic()->IsPaused())
		{
			//game is over, kill our input or CLanlib will freak out later
			KillInput();
		}
		return;
	}

	switch(key.id)
	{
	case CL_KEY_LEFT:
		m_Keys |= C_KEY_LEFT;
		ResetMove(C_DIR_LEFT);
		break;

	case CL_KEY_RIGHT:
		m_Keys |= C_KEY_RIGHT;
		ResetMove(C_DIR_RIGHT);
		break;

	case CL_KEY_UP:
		m_Keys |= C_KEY_UP;
		ResetMove(C_DIR_UP);
		break;

	case CL_KEY_DOWN:
		m_Keys |= C_KEY_DOWN;
		ResetMove(C_DIR_DOWN);
		break;

	case CL_KEY_SPACE:
	case CL_KEY_ENTER:
		CL_Vector2 vDest = GetCurTilePos() + DirStep(m_lastDir);
		
		if (g_Map.IsValid(vDest))
		if (g_Map.GetTile(vDest)->IsEmpty())
		{
			
			if (GetLevelLogic()->CanMakeWood())
			{

				EntObstacle *pObstacle = new EntObstacle();
				pObstacle->Init(g_Map.PosToTileID(vDest), OBSTACLE_WOOD);
				GetEntityManager()->Add(pObstacle);
				g_Sound.Play("build.wav");

			} else
			{
				Say("No Wall Tools!", this, 15);

			}
		} else
		{
			//it's not empty
			
			Tile *pTile = g_Map.GetTile(vDest);
			
			if (pTile->GetObstacle())
			{
				if (GetLevelLogic()->CanMakeWood())
				{
	
				pTile->GetObstacle()->AttemptToDestroy();
				} else
				{
					Say("No Wall Tools!", this, 15);

				}

			} else if (pTile->GetBomb())
			{
				{
					Say("Shit, a bomb!", this, 15);
				}
			}
		}
		break;
	}
}

void EntPlayer::OnKeyUp(const CL_InputEvent &key)
{
	switch(key.id)
	{
	case CL_KEY_LEFT:
		m_Keys &= ~C_KEY_LEFT;
		break;

	case CL_KEY_RIGHT:
		m_Keys &= ~C_KEY_RIGHT;
		break;

	case CL_KEY_UP:
		m_Keys &= ~C_KEY_UP;
		break;

	case CL_KEY_DOWN:
		m_Keys &= ~C_KEY_DOWN;
		break;
	}
}

void EntPlayer::SetSpriteData(CL_Sprite *pSprite)
{
	if (m_spriteLastUsed != pSprite)
	{
		m_sprite->set_image_data(*pSprite);
		m_spriteLastUsed = pSprite;
	}
}

void EntPlayer::SetSpriteData(eDir dir)
{
	if (m_state == STATE_DEAD)
	{
		SetSpriteData(m_spriteDead);
		return;
	}
	
	SetSpriteData(m_spriteDir[dir]);
}

CL_Vector2 EntPlayer::GetCurTilePos()
{
	return g_Map.GetClosestTilePosToThisPixelPoint(m_vPosition);
}

int EntPlayer::GetCurTileID()
{
	return g_Map.PosToTileID(g_Map.GetClosestTilePosToThisPixelPoint(m_vPosition));
}

void EntPlayer::Move(eDir dir)
{
	
	if (m_dirTimer[dir] < GetApp()->GetGameTick())
	{
		m_dirTimer[dir] = GetApp()->GetGameTick()+ C_PLAYER_MOVE_DELAY_MS;

		CL_Vector2 vPos = GetCurTilePos()+DirStep(dir);

		if (g_Map.IsValid(vPos.x, vPos.y))
		{
			Tile *pTile = g_Map.GetTile(vPos.x,vPos.y);
			//let's actually set this as our new destination
			m_destTileID = g_Map.PosToTileID(vPos.x,vPos.y);
		} else
		{
			g_Sound.Play("bump.wav");
		}
	}
}

bool EntPlayer::CloseToTarget()
{
	CL_Vector2 vTarget = g_Map.TileToPixelPos(m_destTileID);

	float fClosenessRequired = 0.13;
	
	if (  (vTarget-m_vPosition).length() < fClosenessRequired) return true;
	return false;
}

void EntPlayer::OnMessage(Message &msg)
{
	switch (msg.m_msg)
	{
	case C_MESSAGE_REINCARNATE:
		
		m_invicibilityTimer = GetApp()->GetGameTick()+1000;
		m_state = STATE_NORMAL;
		m_lastDir = C_DIR_DOWN;

		if (Random(20) == 1)
		{
		int r=Random(4);
		string text;
		switch (r)
		{
			case 0: text = "That was toasty!"; break;
			case 1: text = "Medium rare!"; break;
			case 2: text = "Stop sucking!"; break;
			case 3: text = "Only a flesh wound!"; break;
		}

		Say(text, this, 5);
		}
		break;
	}

	Entity::OnMessage(msg);
}


void EntPlayer::OnDie()
{
	g_Sound.Play("burn.wav");
	QueueSound("die.wav", 70);
	m_destTileID = GetCurTileID(); //die where you stand
	
	
	m_state = STATE_DEAD;
	ResetKeys();

	int r=Random(4);

	string text;
	switch (r)
	{
	case 0: text = "Aaahhh!"; break;
	case 1: text = "Aiiiihhhh!"; break;
	case 2: text = "Fuuuuuukkkkmeeee!"; break;
	case 3: text = "Nooooarghhh!"; break;
	}
	Say(text, this, 5);

	GetGameLogic()->ShowText("Score penalty: -500", 1000);
	GetGameLogic()->ModScore(-500);
	m_lives--;

	if (m_lives > 0)
	{
		GetMessageDispatcher()->SendMsg(C_MESSAGE_REINCARNATE, 1000);
	} else
	{
		GetMessageDispatcher()->SendMsg(C_MESSAGE_INIT_END_LEVEL_TEXT, 2500);
		GetLevelLogic()->SetPaused();
		
	}
	

}

void EntPlayer::UpdateMovement(float step)
{

	if (m_state == STATE_DEAD || GetLevelLogic()->IsPaused()) return;

	if (m_Keys & C_KEY_LEFT) Move(C_DIR_LEFT); else
	if (m_Keys & C_KEY_RIGHT) Move(C_DIR_RIGHT); else
	if (m_Keys & C_KEY_UP) Move(C_DIR_UP); else
	if (m_Keys & C_KEY_DOWN) Move(C_DIR_DOWN);

	//is goal still valid?
	int curTileID  = GetCurTileID();

	Tile *pTile = g_Map.GetTile(m_destTileID);

	if (pTile->GetGirl())
	{
		//well, it isn't empty, but we can still move here
		pTile->GetGirl()->OnRescued();
	} else
	{
		
		if (pTile->GetBomb())
		{
			if (GetLevelLogic()->CanDisarmBombs())
			{
				pTile->GetBomb()->Diffuse();
			} 
		}
		
		if (curTileID != m_destTileID && !pTile->IsEmpty())
		{
			//go back
			//g_Sound.Play("bump.wav");
			m_destTileID = curTileID;
		}
	}
	pTile = g_Map.GetTile(curTileID);

	if (pTile->IsOnFire() && m_invicibilityTimer < GetApp()->GetGameTick())
	{
		OnDie();
	} 
	
	//move towards goal

	CL_Vector2 vDest = g_Map.TileToPixelPos(m_destTileID);

	m_vPosition.x = SetFloatWithTarget(m_vPosition.x, vDest.x, C_MOVE_PIXELS*step);
	m_vPosition.y = SetFloatWithTarget(m_vPosition.y, vDest.y, C_MOVE_PIXELS*step);
}

void EntPlayer::Update(float step)
{
	Entity::Update(step);
	if (step == 0) return; //paused
	UpdateMovement(step);
}

void EntPlayer::Render()
{

	if (m_ignoreKeysUntilTimer > GetApp()->GetGameTick())
	{
		//effect so they can clearly see the player at the start
		int num = ((GetApp()->GetGameTick()%200)/(200/4));
		//LogMsg("Spin is %d", num);
		m_lastDir = eDir(num);
	}
	SetSpriteData(m_lastDir);
	m_sprite->set_alpha(m_alpha);
	m_sprite->set_scale(m_vScale.x, m_vScale.y);

	if (m_invicibilityTimer < GetApp()->GetGameTick() || (GetApp()->GetGameTick())%300 < 150) //flashing when invincible
	{
		m_sprite->draw(m_vPosition.x, m_vPosition.y);
	}

	//also draw lives left
	int x = 700;
	for (int i=0; i < m_lives; i++)
	{
		assert(i < C_DIR_COUNT);
		m_spriteDir[i]->draw(x+=50, 700);
	}
}
