#include "GameLogic.h"
#include "Main.h"
#include "EntShow.h"
#define C_GET_READY_SPEED_MS 600
#include "Map.h"
#include "EntPlayer.h"
#include "Sound.h"

EntPlayer *g_pEntPlayer = NULL;

using namespace std;

GameLogic::GameLogic()
{
	m_slotKeyDown = CL_Keyboard::sig_key_down().connect(this, &GameLogic::OnKeyDown);
	m_levelSelected = 1;
	m_girlsActive = 0;
	g_pEntPlayer = NULL;
	m_bSpaceToInitLevel = false;
	m_showText.clear();

	//load highscores
	FILE *fp = fopen("scores.dat", "rb");
	if (fp)
	{
		fread(m_scoring, sizeof(Scoring), C_LEVEL_COUNT+1, fp);
		fclose(fp);
	}
}

void GameLogic::ClearGame()
{
	m_messageDispatcher.ClearAllMessages();
	
	m_myEntityManager.Init();
	g_pEntPlayer = NULL;
	m_score = 0;
	g_Map.Reset();
	m_bSpaceToInitLevel = true;
	m_showText.clear();
	m_girlsActive = 0;
	
}

GameLogic::~GameLogic()
{
	FILE *fp = fopen("scores.dat", "wb");
	if (fp)
	{
		fwrite(m_scoring, sizeof(Scoring), C_LEVEL_COUNT+1, fp);
		fclose(fp);
	}


	m_messageDispatcher.ClearAllMessages();
    Kill();
   
}


void GameLogic::OnMessage(Message &msg)
{
    switch(msg.m_msg)
    {
    case C_MESSAGE_SOUND:
		g_Sound.Play(msg.m_text);
        break;
	
	case C_MESSAGE_ADD_TILE:
		{
			switch(msg.m_parm)
			{
				case LevelLogic::TILE_ROCK:
				{
					EntObstacle *pEnt = new EntObstacle;
					pEnt->Init(msg.m_parm1, OBSTACLE_ROCK);
					GetEntityManager()->Add(pEnt);
					break;
				}
				case LevelLogic::TILE_WATER:
					{
						EntObstacle *pEnt = new EntObstacle;
						pEnt->Init(msg.m_parm1, OBSTACLE_WATER);
						GetEntityManager()->Add(pEnt);
						break;
					}
				case LevelLogic::TILE_WOOD:
					{
						EntObstacle *pEnt = new EntObstacle;
						pEnt->Init(msg.m_parm1, OBSTACLE_WOOD);
						GetEntityManager()->Add(pEnt);
						break;
					}
				case LevelLogic::TILE_GIRL:
					{
						EntGirl *pEnt = new EntGirl;
						pEnt->Init(msg.m_parm1);
						GetEntityManager()->Add(pEnt);
						break;
					}

				case LevelLogic::TILE_PLAYER_START:
					g_pEntPlayer = new EntPlayer();
					g_pEntPlayer->SetPosTile(msg.m_parm1);
					GetEntityManager()->Add(g_pEntPlayer);

					break;

					
			}
		}
		break;
	case C_MESSAGE_TEXT:
		{
			EntShow *pEnt = new EntShow();
			
			if (msg.m_pParmEnt)
			{
				msg.m_parmVec = msg.m_pParmEnt->GetPos()+CL_Vector2(C_TILE_SIZE/2, -25);
			}

			if (msg.m_parmVec.x < 64)
			{
				msg.m_parmVec.x = 64;
			}

			pEnt->SetupText(msg.m_text, msg.m_parmVec, eText(msg.m_parm1));
			
			m_myEntityManager.Add(pEnt);
			
			int timeToLive = max(1700, msg.m_text.size()*50);
			

			if (msg.m_parm != 0) timeToLive = msg.m_parm;
			GetMessageDispatcher()->SendMsg(C_MESSAGE_FADE_OUT_AND_DIE, timeToLive, pEnt);
		}
		break;

	case C_MESSAGE_INIT_END_LEVEL_TEXT:
		if (m_state == STATE_PLAYING)
		{
			m_showText = m_levelLogic.GetSummary();
			g_Sound.Play("whirly.wav");

		}
		
		break;

	case C_MESSAGE_RESTART:
		StartGame();
		break;
	}
}

void GameLogic::ShowText(string text, int timeMS, int x, int y)
{
	m_messageDispatcher.SendText(text, timeMS, 0, CL_Vector2(x,y));
}


void GameLogic::SetupTitleScreen()
{
	ClearGame();
	GetApp()->SetupBackground("media/title.jpg", GetApp()->GetMainWindow()->get_width(), GetApp()->GetMainWindow()->get_height());
	m_state = STATE_TITLE_SCREEN;
}

void GameLogic::StartGame()
{
	m_state = STATE_PLAYING;
	GetApp()->SetupBackground("media/game_bg.jpg",0,0);
	ClearGame();

	g_Sound.Play("warning.wav");
	m_levelLogic.Init(m_levelSelected);
}


void GameLogic::OnKeyDown(const CL_InputEvent &key)
{
       
	
	if (m_state == STATE_TITLE_SCREEN)
	{
		switch (key.id)
		{
			
			//these look weird because I stupidly make the level select 1-based for some reason
		case CL_KEY_DOWN:
			m_levelSelected = (m_levelSelected%C_LEVEL_COUNT)+1;
			break;
			
		case CL_KEY_UP:
				m_levelSelected = m_levelSelected-1;
				if (m_levelSelected == 0) m_levelSelected = C_LEVEL_COUNT;
				break;
		}
	}
	
	
	
	switch (key.id)
        {
		 case CL_KEY_SPACE:
		 case CL_KEY_ENTER:
		 
			 switch (m_state)
			 {
			 case STATE_TITLE_SCREEN:
				 StartGame();
				 break;

			 case STATE_PLAYING:
				if (m_bSpaceToInitLevel)
				{
					m_levelLogic.InitLevelData();
					m_bSpaceToInitLevel = false;
				}	
				
				if (!m_showText.empty() )
				{
					//space to retry
					GetMessageDispatcher()->SendMsg(C_MESSAGE_RESTART, 150);
				}
					
				
				break;
			 }
			 break;
        
		 
		 case  CL_KEY_F11:
            break;

		 case CL_KEY_ESCAPE:

			 switch (m_state)
			 {
			 case STATE_TITLE_SCREEN:
				 GetApp()->OnWindowClose();
				 break;

			 case STATE_PLAYING:
					SetupTitleScreen();
				 break;
			 }
			 break;
        }
}

void GameLogic::RebuildBuffers()
{
    //note, currently, this can get called before Init() is called!
 //   GetApp()->GetBackgroundCanvas()->get_gc()->clear(CL_Color(0,255,255));
}

void GameLogic::Kill()
{
}

void GameLogic::Update(float step)
{
    m_messageDispatcher.Update();  
    m_myEntityManager.Update(step);
	m_levelLogic.Update(step);
	g_Map.Update(step);

	if (m_state != STATE_TITLE_SCREEN)
	{
		} else
	{
	
		//for fun, why not spawn some bombs

		static int thinkTimer = 0;
		if (thinkTimer < GetApp()->GetGameTick())
		{
			thinkTimer = GetApp()->GetGameTick() + 500;
			if (Random(2) == 0)
				GetLevelLogic()->SpawnBomb();

		}
	}
}


void GameLogic::ModScore(int mod)
{
	m_score += mod;
}

void GameLogic::RenderGUI()
{
	GetApp()->GetFont(C_FONT_MAIN)->set_alignment(origin_top_left);
	GetApp()->GetFont(C_FONT_MAIN)->draw(10, 640, "Score: "+CL_String::from_int(m_score));
	GetApp()->GetFont(C_FONT_MAIN)->draw(10, 710, "Mission: "+m_levelLogic.GetLevelName());

	GetApp()->GetFont(C_FONT_MAIN)->draw(800, 640, "Girls Left: "+CL_String::from_int(GetGirlsLeft()));
	
}

void GameLogic::Render()
{
    CL_Display::clear(CL_Color(0,0,255));
	GetApp()->GetBackground()->draw(0,0);
	m_myEntityManager.Render();    
	g_Map.Render();

	if (m_state != STATE_TITLE_SCREEN)
	{
	
		if (m_bSpaceToInitLevel)
		{

			CL_Rect r = CL_Rect(0,0,GetScreenX-0, GetScreenY-0);
			CL_Display::fill_rect(r, CL_Color(0,0,0,100));

			GetApp()->GetFont(C_FONT_MAIN)->set_alignment(origin_center);
			GetApp()->GetFont(C_FONT_MAIN)->draw(500, 350, m_levelLogic.GetLevelStartText());
		}

		if (!m_showText.empty())
		{
			CL_Rect r = CL_Rect(0,0,GetScreenX-0, GetScreenY-0);
			CL_Display::fill_rect(r, CL_Color(0,0,0,100));
			//must be at round end
			GetApp()->GetFont(C_FONT_MAIN)->set_alignment(origin_center);
			GetApp()->GetFont(C_FONT_MAIN)->draw(500, 350, m_showText);

		}

		RenderGUI();
	} else
	{

		DrawMenuSelect();

	}

	
}

MyEntityManager * GetEntityManager()
{
	return GetGameLogic()->GetMyEntityManager();
}

MessageDispatcher * GetMessageDispatcher()
{
	return GetGameLogic()->GetMessageDispatcher();
}

GameLogic * GetGameLogic()
{
	return GetApp()->GetGameLogic();
}

void QueueSound(string soundFile, int delayMS)
{
	Message m;
	m.m_text = soundFile;
	m.m_msg = C_MESSAGE_SOUND;
	GetMessageDispatcher()->SendMsgEx(m, delayMS);
}

void Say(string text, Entity *pEnt, int delayMS)
{
	Message m;
	m.m_text = text;
	m.m_pParmEnt = pEnt;
	m.m_msg = C_MESSAGE_TEXT;
	m.m_parm = 0; //delay time
	m.m_parm1 = TEXT_ENTITY;
	
	
	GetMessageDispatcher()->SendMsgEx(m, delayMS);
}

LevelLogic * GetLevelLogic()
{
	return GetGameLogic()->GetLevelLogic();
}

void GameLogic::DrawMenuSelect()
{
	GetApp()->GetFont(C_FONT_MAIN)->set_alignment(origin_center);

	string name;

	int x = 600;
	int y = 300;

	for (int i=0; i < C_LEVEL_COUNT; i++)
	{
		//first get the data
		m_levelLogic.Init(i+1);
		
		name = m_levelLogic.GetLevelName();
		string extra;
		if (m_scoring[i+1].m_passed)
		{
			extra = " [PASSED] ";
		}

		name += extra;

		if (i+1 == m_levelSelected)
		{

			//cheap ass anim effect 


			if (GetApp()->GetGameTick()%300 < 150)
			{
				name = "* "+name+" *";
			} else
			{
				name = "*  "+name+"  *";
			}
		}

		GetApp()->GetFont(C_FONT_MAIN)->draw(x, y, name);

		y += 31;


	}

	m_levelLogic.Init(m_levelSelected);

	name = m_levelLogic.GetLevelName()+" Highscore: ";
	
	if (m_scoring[m_levelSelected].m_highscore == -999 )
	{
		name += "None yet!";
	} else
	{
		name += CL_String::from_int(m_scoring[m_levelSelected].m_highscore);
	}
	
	GetApp()->GetFont(C_FONT_MAIN)->draw(x, 650, name);


}
