#include "Map.h"
#include "GameLogic.h"
#include "EntPlayer.h"

int const C_FIRE_ANIM_SPEED_MS = 100;

Map g_Map;

Map::Map()
{
	m_pFire = NULL;
}

Map::~Map()
{
	SAFE_DELETE(m_pFire);
}

Tile * Map::GetTile(int tileID)
{
	assert (tileID >= 0 && tileID < C_TOTAL_TILES_IN_MAP && "Dumbass!");
	return &m_tiles[tileID];
}

Tile * Map::GetTile(CL_Vector2 vTilePos)
{
	return GetTile(PosToTileID(vTilePos));
}

bool Map::IsValid(int x, int y)
{
	if (x < 0 || x >= C_MAP_SIZE_X) return false;
	if (y < 0 || y >= C_MAP_SIZE_Y) return false;
	return true;
}

bool Map::IsValid(CL_Vector2 v)
{
	return IsValid(v.x, v.y);
}

Tile * Map::GetTile( int x, int y )
{
	return &m_tiles[PosToTileID(x,y)];
}

int Map::PosToTileID(int x, int y)
{
	return y*C_MAP_SIZE_X + x;
}

int Map::PosToTileID(CL_Vector2 vPos)
{
	return int(vPos.y)*C_MAP_SIZE_X + int(vPos.x);
}

int Map::GetEmptyTile(int requiredDistanceFromPlayer)
{
	int tries = 0;
	int tileID;

	while (tries++ < 200)
	{
		
		tileID = Random(C_TOTAL_TILES_IN_MAP);
		if (GetTile(tileID)->IsEmpty())
		{
			if (requiredDistanceFromPlayer != 0 && g_pEntPlayer)
			{
				if (GetDistanceToPlayer(tileID) <= requiredDistanceFromPlayer)
				{
					//nah, we're too close to the player				
					continue;
				}
					
			}

			return tileID;
		}
	}
	tileID = -1;
	LogMsg("Failed to find valid tile");
	return tileID;
}


CL_Vector2 Map::TileToPos( int tileID )
{

	CL_Vector2 v = CL_Vector2( int(tileID%C_MAP_SIZE_X), int(tileID/C_MAP_SIZE_X));
	//LogMsg("TileID %d converts to %d,%d", tileID, int(v.x), int(v.y));
	return v;
}

void Map::Reset()
{
	if (!m_pFire)
	{
		m_pFire = new CL_Sprite("fire_anim", GetApp()->GetResourceManager());
	}
	
	for (int i=0; i < C_TOTAL_TILES_IN_MAP; i++)
		m_tiles[i].Reset(i);
}

CL_Vector2 Map::TileToPixelPos( int tileID )
{
	return TileToPos(tileID)*C_TILE_SIZE;
}

CL_Vector2 Map::TileToPixelPosCentered( int tileID )
{
	CL_Vector2 v = TileToPixelPos(tileID);
	v.x += C_TILE_SIZE/2;
	v.y += C_TILE_SIZE/2;
	return v;
}

void Tile::SetOnFire()
{
	if (m_pObstacle)
	{
		if (!m_pObstacle->IsBurnable())
		{
			return;
		}
	}
	
	m_bIsOnFire = true;
	m_curFrame = 0;
	m_timeSetAblaze = GetApp()->GetGameTick();
	m_timeOfNextFrame = GetApp()->GetGameTick() + C_FIRE_ANIM_SPEED_MS;
}

void Map::Update(float step)
{
	for (int i=0; i < C_TOTAL_TILES_IN_MAP; i++)
		m_tiles[i].Update(step);
}

void Map::Render()
{
	for (int i=0; i < C_TOTAL_TILES_IN_MAP; i++)
		m_tiles[i].Render();
}

int  Map::GetDistanceToPlayer(int tileID)
{
	if (!g_pEntPlayer) return 100;
	return (TileToPos(tileID)-g_pEntPlayer->GetCurTilePos()).length();
}

CL_Vector2 Map::GetClosestTilePosToThisPixelPoint( CL_Vector2 vPos )
{
	CL_Vector2 v;

	v.x = (int(vPos.x) + C_TILE_SIZE/2) / C_TILE_SIZE;
	v.y = (int(vPos.y) + C_TILE_SIZE/2) / C_TILE_SIZE;
	return v;
}

void Tile::Render()
{
	if (m_bIsOnFire)
	{
		CL_Vector2 v = g_Map.TileToPixelPos(m_tileID);
		g_Map.m_pFire->set_frame(m_curFrame);
		g_Map.m_pFire->draw(v.x,v.y);
	}
}

void Tile::PutOutFire()
{
	m_bIsOnFire = false;
}

void Tile::Update( float step )
{
	if (m_bIsOnFire)
	{
		if (m_pObstacle)
		{
			m_pObstacle->BurnedByFire();
		}

		if (m_timeOfNextFrame < GetApp()->GetGameTick())
		{
			m_timeOfNextFrame = GetApp()->GetGameTick() + C_FIRE_ANIM_SPEED_MS;
			m_curFrame++;
			if (m_curFrame == 5)
			{
				//uh, no such frame.  I guess we're done being on fire, huh?
				PutOutFire();
			}
		}
	}
}

void Tile::SetObstacle( EntObstacle *pEnt )
{
	assert(!m_pObstacle || !pEnt);
	m_pObstacle = pEnt;
}

bool Tile::IsEmpty()
{
		if (g_pEntPlayer && m_tileID == g_pEntPlayer->GetCurTileID())
		{
			return false;
		}
	
	
		if (m_pBomb) return false;
		if (m_pObstacle) return false;
		if (m_pGirl) return false;

		
		return true;
}

void Tile::SetGirl( EntGirl *pGirl )
{
	assert(!m_pGirl || !pGirl);
	m_pGirl = pGirl;
}