//  ***************************************************************
//  Map - Creation date: 12/15/2007
//  -------------------------------------------------------------
//  Robinson Technologies Copyright (C) 2007 - All Rights Reserved
//
//  ***************************************************************
//  Programmer(s):  Seth A. Robinson (seth@rtsoft.com)
//  ***************************************************************

#ifndef Map_h__
#define Map_h__


#include "main.h"
#include "EntBomb.h"
#include "EntObstacle.h"
#include "EntGirl.h"

const int C_MAP_SIZE_X = 16;
const int C_MAP_SIZE_Y = 10;

const int C_TOTAL_TILES_IN_MAP = C_MAP_SIZE_X*C_MAP_SIZE_Y;
const int C_TILE_SIZE = 64;

class Tile
{
public:

	Tile()
	{
		Reset(-1);
	}

	void Reset(int tileID)
	{
		m_pBomb = NULL;
		m_pObstacle = NULL;
		m_bIsOnFire = false;
		m_curFrame = 0;
		m_tileID = tileID;
	}
	
	bool IsEmpty();
	

	void SetBomb(EntBomb *pBomb)
	{
		assert(!m_pBomb);
		m_pBomb = pBomb;
	}
	
	EntBomb * GetBomb() {return m_pBomb;};

	void RemoveBomb()
	{
		m_pBomb = NULL;
	}

	void SetOnFire();
	void Render();
	void Update(float step);
	void PutOutFire();
	void SetObstacle(EntObstacle *pEnt);
	EntObstacle * GetObstacle() {return m_pObstacle;};
	bool IsOnFire() {return m_bIsOnFire && m_curFrame < 3;}

	void SetGirl(EntGirl *pGirl);
	EntGirl * GetGirl() {return m_pGirl;}

private:

	bool m_bIsOnFire;
	EntBomb *m_pBomb;
	EntObstacle *m_pObstacle;
	EntGirl *m_pGirl;
	int m_curFrame;
	int m_timeSetAblaze;
	int m_tileID;
	int m_timeOfNextFrame;



};


class Map
{
public:
	Map();
	virtual ~Map();

	int GetEmptyTile(int requiredDistanceFromPlayer=0);
	int PosToTileID(int x, int y);
	int PosToTileID(CL_Vector2 vPos);
	CL_Vector2 TileToPos(int tileID);
	CL_Vector2 TileToPixelPos(int tileID);
	CL_Vector2 TileToPixelPosCentered(int tileID);
	CL_Vector2 GetClosestTilePosToThisPixelPoint(CL_Vector2 vPos);
	Tile * GetTile(int tileID);
	Tile * GetTile(int x, int y);
	Tile * GetTile(CL_Vector2 vTilePos);
	void Reset();
	bool IsValid(int x, int y);
	bool IsValid(CL_Vector2 v);
	void Render();
	void Update(float step);
	int GetDistanceToPlayer(int tileID);
	friend Tile;

private:

	CL_Sprite * m_pFire;
	Tile m_tiles[C_TOTAL_TILES_IN_MAP];
};



extern Map g_Map;

#endif // Map_h__